/* rover.cpp
 * 
 * Supporting library for the trinket rover.
 * Includes servo control as well as some other functions.
 */
 
#include "Arduino.h"
#include "rover.h"

/* no more macro in this file!
 * otherwise nasty semi-recursive from the return.
 */
#undef analogRead(pin)

int trinketAnalogRead(uint8_t pin) {
	/* Convert real pin # to analog pin number.
	 * -1 means the pin is not analog...
	 *
	 * analog pin 3 (on PB3) *is* an analog input (3),
	 * but it has a USB pull-up. So it doesn't work.
	 */
	 
	uint8_t realToAnalog[] = {-1, -1, 1, -1, 2};
	
	if(realToAnalog[pin] < 0) {
		// can't do analog read on that pin!
		return -1;
	}
	
	return analogRead(realToAnalog[pin]);
}



/* ==== Servo Control ==== */

uint8_t leftPin, rightPin;
int8_t leftSpeed, rightSpeed;

/* 0 => 1
 * speed ratio for servo calibration
 */
float leftCal = 1;
float rightCal = 1;

/* This sets up the timers and such. */
void rover(uint8_t _leftPin, uint8_t _rightPin) {
	leftPin = _leftPin;
	rightPin = _rightPin;
	
	pinMode(leftPin, OUTPUT);
	pinMode(rightPin, OUTPUT);
	
	cli();

	/* Turn on interrupts */
	sbi(TIMSK, OCIE1A); // leftPin servo
	sbi(TIMSK, OCIE1B); // rightPin servo

	sbi(TIMSK, TOIE1); // Overflow, for overall control 

	stop();
	
	sei();
	
	// Default values
	leftSpeed = 187 + 63 * leftCal;
	rightSpeed = 187 - 62 * rightCal;

}

void start() {
	/* Turn on servos and start timer. */
	
	sbi(PORTB, leftPin);
	sbi(PORTB, rightPin);

	// 64 Prescale
	sbi(TCCR1, CS12); 
	sbi(TCCR1, CS11); 
	sbi(TCCR1, CS10);
	cbi(TCCR1, CS13);

	OCR1A = leftSpeed * leftCal;
	OCR1B = rightSpeed * rightCal;

	TCNT1 = 0;
}

void stop() {
	TCCR1 &= ~B1111; // turn off timer (and thus servo update)
}

void straight(int8_t speed) {
	/* speed is -62 -> 63 */
	leftSpeed = 187 + speed * leftCal;
	rightSpeed = 187 - speed * rightCal;
}


void rotate(int8_t speed) {
	/* Positive is clockwise,
	 * Negative is not.
	 * (range -62 to 63)
	 *
	 * To be clear — this is (or should be) a rotation about the center.
	 */
	 
	 leftSpeed = 187 + speed * leftCal;
	 rightSpeed = 187 + speed * rightCal;
}

void turnL(int8_t speed) {
	/* Turn: left motor stops & right goes.
	 * -62 -> 63
	 */
	
	leftSpeed = 187;
	rightSpeed = 187 - speed * rightCal;
}

void turnR(int8_t speed) {
	/* Turn: right motor stops & left goes.
	 * -62 -> 63
	 */
	
	rightSpeed = 187;
	leftSpeed = 187 + speed * leftCal;
}

void cal(float left, float right) {
	/* Allow for calibration of robot drive.
	 * left, right <= 1
	 * sets ratio of requested speed to actual speed
	 * (use this function to *slow down* the faster motor)
	 */
	leftCal = constrain(left, 0, 1);
	rightCal = constrain(right, 0, 1);
}
	



/* ============== ISR ==============
 * Interrupt Routines below for Servo Control
 */

ISR(TIMER1_COMPA_vect) {
  /* Either:
   *  - Left servo should go low, or
   *  - 20 ms period has finished (both servos high)
   */

  if (bit_is_set(PORTB, leftPin)) {
    /* Servo is high - turn it off. */
    cbi(PORTB, leftPin);
  } 
  else {
    /* Servo is off - 20ms has finished.
     * Turn both servos on & reset clocks.
     */
    sbi(PORTB, leftPin);
    sbi(PORTB, rightPin);

    // 64 Prescale
    sbi(TCCR1, CS12); 
    sbi(TCCR1, CS11); 
    sbi(TCCR1, CS10);
    cbi(TCCR1, CS13);
    
    OCR1A = leftSpeed;
    OCR1B = rightSpeed;
    
    TCNT1 = 0;
  }
}

ISR(TIMER1_COMPB_vect) {
  /* Right servo's period is up. Turn off. */
  cbi(PORTB, rightPin);
}

ISR(TIMER1_OVF_vect) {
  /* Both servos are LOW at this point.
   * Engage 20ms wait-period on timer1A.
   */

  // 1024 Prescale
  sbi(TCCR1, CS13); 
  sbi(TCCR1, CS11); 
  sbi(TCCR1, CS10);
  cbi(TCCR1, CS12);

  // This may need some adjustment.
  OCR1A = 157;
}